/** \file Atod.c
*
* Allows use of the onboard AtoD in the TI MSP430 target part
*
* $Name:  $
*/
/*******************************************************************************
* Copyright (c) 2006 Dynastream Innovations Inc.
* THE FOLLOWING EXAMPLE CODE IS INTENDED FOR LIMITED CIRCULATION ONLY.
* 
* Please forward all questions regarding this code to ANT Technical Support.
* 
* Dynastream Innovations Inc.
* 228 River Avenue
* Cochrane, Alberta, Canada
* T4C 2C1
* 
* (P) (403) 932-9292
* (F) (403) 932-6521
* (TF) 1-866-932-9292
* (E) support@thisisant.com
* 
* www.thisisant.com
*
*******************************************************************************/

#include "Atod.h"
#include "Config.h"
#include "timer.h"
#include "antmessage.h"
#include "antdefines.h"

//-------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// AtoD_Init
///
/// This function is used to initialize the AtoD to sample the internal temperature sensor
///
/////////////////////////////////////////////////////////////////////////////
void AtoD_Init(void)
{
   SD16CTL   |= SD16DIV_3 | SD16XDIV_1;	                  // Set the sample clock divider
   SD16CCTL0  = SD16UNI | SD16SNGL | SD16OSR_1024;        // Set unipolar range (0 to max value), single conversion mode, 1024x oversampling rate
   SD16INCTL0 = SD16INCH_6;                               // Select Temperature Sensor as input channel
}	

//-------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// AtoD_GetSample
///
/// This function is used to sample the internal temperature sensor.  
///
/// \return The return value is the 16-bit AtoD value read from the temperature sensor.
///
/////////////////////////////////////////////////////////////////////////////
USHORT AtoD_GetSample(void)
{
   SD16CTL |= SD16REFON;		                  // turn on the AtoD reference
   Timer_DelayTime(10);                                   // Let reference settle

   SD16CCTL0 |= SD16SC;                                   // initiate a sample
   while((SD16CCTL0 & SD16SC));                           // wait for sample to finish

   SD16CCTL0 &= ~SD16IFG;                                 // clear sample flag
   SD16CTL   &= ~SD16REFON;                               // turn off the AtoD reference

   return SD16MEM0;			                  // return the sample value
}

//-------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// AtoD_SetTempMesg
///
/// This function is used to convert the AtoD reading into a termperature value
/// and build the message payload to send over RF
///
/// \param pucTxMesg - Pointer to buffer where the Tx message should be written
/// \param ucChannel - The ANT channel the message is ment for
///
/////////////////////////////////////////////////////////////////////////////
void AtoD_SetTempMesg(UCHAR *pucTxMesg, UCHAR ucChannel)
{
   //Temperature conversion for this particular AtoD and built in temperature sensor
   //Temperature (1/100 Degrees C) = (((AtoD Value/65536)*600)/1.32 - 273) * 100;     
  
   ULONG ulAtoDVal = AtoD_GetSample();                    // Get A/D Sample (Temperature Sensor Reading)
   
   ulAtoDVal = ((ulAtoDVal * 45455) >> 16) - 27300;       // The equation above rearranged

   *pucTxMesg++ = MESG_DATA_SIZE;                         // set the data message size
   *pucTxMesg++ = MESG_BROADCAST_DATA_ID;                 // set the message ID
   *pucTxMesg++ = ucChannel;                              // set the channel number
   *pucTxMesg++ = 0;                                      // reserved 
   *pucTxMesg++ = 0;                                      // reserved
   *pucTxMesg++ = SHARED_TYPE_A_TO_D;                     // define as a standard temperature sensor message
   *pucTxMesg++ = TEMPERATURE;                            // AtoD Subtype 
   *pucTxMesg++ = 0;                                      // reserved
   *pucTxMesg++ = 0;                                      // reserved
   *pucTxMesg++ = (UCHAR)ulAtoDVal;                       // temperature (USHORT)lsb
   *pucTxMesg   = (UCHAR)(ulAtoDVal >> 8);                // temperature (USHORT)msb
}
