/** \file BitSyncSerial.c
*
* Implements the Bit Synchronous Serial Port
*
* $Name:  $
*/
/******************************************************************************
* Copyright (c) 2006 Dynastream Innovations Inc.
* THE FOLLOWING EXAMPLE CODE IS INTENDED FOR LIMITED CIRCULATION ONLY.
* 
* Please forward all questions regarding this code to ANT Technical Support.
* 
* Dynastream Innovations Inc.
* 228 River Avenue
* Cochrane, Alberta, Canada
* T4C 2C1
* 
* (P) (403) 932-9292
* (F) (403) 932-6521
* (TF) 1-866-932-9292
* (E) support@thisisant.com
* 
* www.thisisant.com
*******************************************************************************/

#include "types.h"
#include "config.h"
#include "antmessage.h"
#include "timer.h"      // required for timing purposes


#define SEN_ASSERT_TIMEOUT             ((UCHAR)3)                                   ///< wait 30us for SEN to be asserted in response to SMSGRDY assertion                   

#define IS_SEN_ASSERTED()              ((SYNC_SEN_IN & SYNC_SEN_BIT) == 0)          ///< active low

#define IS_SMSGRDY_ASSERTED()          ((SYNC_SMSGRDY_OUT & SYNC_SMSGRDY_BIT) == 0) ///< active low
#define SYNC_SMSGRDY_ASSERT()          (SYNC_SMSGRDY_OUT &= ~SYNC_SMSGRDY_BIT)      ///< active low
#define SYNC_SMSGRDY_DEASSERT()        (SYNC_SMSGRDY_OUT |= SYNC_SMSGRDY_BIT)

#define SYNC_SCLK_EDGE_DETECT()        (SYNC_SCLK_IFG & SYNC_SCLK_BIT)              ///< this must be an edge interrupt flag
#define SYNC_SCLK_EDGE_CLEAR()         (SYNC_SCLK_IFG  &= ~SYNC_SCLK_BIT)           ///< clear the interrupt flag

#define SYNC_SRDY_ASSERT()             (SYNC_SRDY_OUT &= ~SYNC_SRDY_BIT)            ///< active low
#define SYNC_SRDY_DEASSERT()           (SYNC_SRDY_OUT |= SYNC_SRDY_BIT)

/// this is a delay of at least 2.5us
#define SYNC_SRDY_PULSE()              { SYNC_SRDY_ASSERT();\
                                         TIMER_SRDY_PULSE_DELAY();\
                                         SYNC_SRDY_DEASSERT(); }                    


volatile BOOL bSCLK_Detected = FALSE;                                               ///< Flag used to indicate if a SCLK edge has been detected

//------------------------------------------------------------------------------


// local functions
static UCHAR ReadByte(void);
static void  WriteByte(UCHAR ucByte);


//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// BitSyncSerial_Init
///
/// This function is used initialize the pins and interrupts used to communicate
/// with the ANT chip
///
/////////////////////////////////////////////////////////////////////////////
void BitSyncSerial_Init(void)
{
   // Serial Lines Intialization
   // SEN
   SYNC_SEN_DIR &= ~SYNC_SEN_BIT;                     // set the direction to be an input

   // SMSGRDY
   SYNC_SMSGRDY_DEASSERT();                           // make sure it is deasserted so that it doesn't misguide ANT
   SYNC_SMSGRDY_DIR |= SYNC_SMSGRDY_BIT;              // set SMSGRDY as an output

   // SIN
   SYNC_SIN_DIR &= ~SYNC_SIN_BIT;                     // set SIN as an input

   // SOUT
   SYNC_SOUT_DIR |= SYNC_SOUT_BIT;                    // set SOUT as an output

   // SCLK
   SYNC_SCLK_DIR &= ~SYNC_SCLK_BIT;                   // set SCLK as an input
   SYNC_SCLK_IES &= ~SYNC_SCLK_BIT;                   // interrupt on the low-to-high transition
   SYNC_SCLK_IFG &= ~SYNC_SCLK_BIT;                   // clear the interrupt flag		
   SYNC_SCLK_IE |= SYNC_SCLK_BIT;                     // Enable the intterupt

   // SRDY
   SYNC_SRDY_DEASSERT();                              // put SRDY in a known state
   SYNC_SRDY_DIR |= SYNC_SRDY_BIT;                    // set SRDY as an output
}

//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// BitSyncSerial_SyncReset
///
/// This function performs the synchronous reset sequence for the ANT chip
///
/// \return Boolean value inidcating weather or not the ANT chip was successfully reset
/////////////////////////////////////////////////////////////////////////////
BOOL BitSyncSerial_SyncReset(void)
{   
   SYNC_SMSGRDY_DEASSERT();
   // Add delay to wait for ANT module to finish power up and get ready for incoming messages
   // The current system setup results in a delay of ~1ms; a minimum of 0.5ms 
   // are required for ANT to properly reset.
   Timer_DelayTime(100);                               // 1 ms

   // perform the Synchronous serial reset as described in Interfacing with
   // ANT General Purpose Chipsets and Modules document

   // Assert SRDY (SRDY = 0)
   SYNC_SRDY_ASSERT();

   // make sure there is a 250us delay between SRDY falling edge and SMSGRDY falling edge
   Timer_DelayTime(30);                                // 300 us delay

   // Assert SMSGRDY (SMSGRDY = 0)
   SYNC_SMSGRDY_ASSERT();

   // Wait for ANT to lower SEN to indicate it is ready
   Timer_DelayTime(100);                               // wait 1 ms

   // raise SRDY (SRDY = 1)
   SYNC_SRDY_DEASSERT();

   return IS_SEN_ASSERTED();
}

//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// BitSyncSerial_Transaction
///
/// This function is used to check for message from the ANT chip as well as send
/// messages to the ANT chip.  In the case where there is both a message to transmit
/// and recieve, it will first receive the message from the ANT chip then send the 
/// pending transmit message.  
///
/// \param[in] pucTxMsgBuffer - pointer to the buffer that holds the message to be transmitted, if no message is to be transmittd this pointer can be set to NULL
/// \param[out] pucRxMsgBuffer - pointer to the buffer where a received message will be written
/// \return Boolean value indicating weather a message was received from the ANT chip
/////////////////////////////////////////////////////////////////////////////
BOOL BitSyncSerial_Transaction(UCHAR *pucTxMsgBuffer, UCHAR *pucRxMsgBuffer)
{
   BOOL  bRxMessage = FALSE;
   UCHAR ucCheckSum;
   UCHAR ucMesgLen;
   UCHAR ucIndex;
   UCHAR ucByte;
   
   if (pucTxMsgBuffer != NULL)                        // if we have a message to send
   {  
      SYNC_SMSGRDY_ASSERT();                          // notify the ANT device that the Host wishes to send a message
      
      // the following retry loop is a workaround to avoid a possible loss of the MSGRDY assertion edge with the nRF24AP1
      {
         UCHAR ucSenTimeout = SEN_ASSERT_TIMEOUT;

         do                                              
         {
            if (!(ucSenTimeout--))                    // check the retry timeout
            {
               ucSenTimeout = SEN_ASSERT_TIMEOUT;     // Reset the timeout for the SEN response 

               SYNC_SMSGRDY_DEASSERT();               // Pulse MSGRDY agian if SEN has not been asserted by the timeout
               Timer_DelayTime(1);                    
               SYNC_SMSGRDY_ASSERT();            
            }
          
            Timer_DelayTime(1);                       // wait 10us   
         }
         while (!IS_SEN_ASSERTED());                  // check if SEN is asserted
      }
   }            
   
   while (IS_SMSGRDY_ASSERTED() || (IS_SEN_ASSERTED() && (!bRxMessage))) // while we have a message to send or a message to receive when we haven't already received one
   {   
      while (!IS_SEN_ASSERTED());                     // make sure SEN is asserted
     
      // Read the sync Byte 
      ucByte     = ReadByte();                        // read the sync byte
      ucCheckSum = ucByte;                            // initialize the checksum

      if (ucByte == MESG_RX_SYNC)                     // ANT is ready to receive our message
      {
         SYNC_SMSGRDY_DEASSERT();                     // clear the message ready signal

         ucMesgLen = pucTxMsgBuffer[0] + MESG_SAVED_FRAME_SIZE; // read the message length of the message to send and adjust for frame size
         ucIndex   = 0;

         do
         {
            WriteByte(pucTxMsgBuffer[ucIndex]);       // write out message byte
            ucCheckSum ^= pucTxMsgBuffer[ucIndex];    // update the checksum

         } while (++ucIndex < ucMesgLen);             // loop through message

         WriteByte(ucCheckSum);                       // write out the checksum byte
      }
      else if (ucByte == MESG_TX_SYNC)                // ANT wants to send us a message
      {
         // Read the second byte which is the length of the message 
         ucByte    = ReadByte();
         ucMesgLen = ucByte + MESG_SAVED_FRAME_SIZE;  // save the message length and correct for the frame size
         ucIndex   = 0;

         do
         {
            pucRxMsgBuffer[ucIndex] = ucByte;         // save it in the receive buffer            
            ucCheckSum ^= ucByte;                     // update the checksum
            ucByte      = ReadByte();                 // read out the next byte

         } while (++ucIndex < ucMesgLen);

         if (ucCheckSum == ucByte)                    // validate the checksum of the message
            bRxMessage = TRUE;                        // flag that we have received a message
      }
   } 

   return bRxMessage;                                 // return whether we received a message or not
}

//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// ReadByte
///
/// This function is used to read a single byte from the ANT chip
///
/// \return The byte read from the ANT chip
/////////////////////////////////////////////////////////////////////////////
UCHAR ReadByte(void)
{
   UCHAR ucByte    = 0;
   UCHAR ucBitMask = 0x01;   
   bSCLK_Detected = FALSE;                            // Clear the interrupt status flag

   do
   {
      // Pulse to SRDY to show that we are ready for another bit
      SYNC_SRDY_PULSE();
   
      // Wait for the SCLK rising edge
      while (!bSCLK_Detected);
      bSCLK_Detected = FALSE;                         // Clear the interrupt status flag
	
      // Read the bit
      if (SYNC_SIN_IN & SYNC_SIN_BIT)
         ucByte |= ucBitMask;

      ucBitMask <<= 1;

   } while (ucBitMask);	                              // Read the entire byte

   return (ucByte);
}

//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// WriteByte
///
/// This function is used to write a single byte to the ANT chip
///
/// \param ucByte - The byte to write
/////////////////////////////////////////////////////////////////////////////
void WriteByte(UCHAR ucByte)
{
   UCHAR ucBitMask = 0x01;
   bSCLK_Detected = FALSE;                            // Clear the interrupt status flag

   do
   {
      // Write the bit
      if (ucByte & ucBitMask)
         SYNC_SOUT_OUT |= SYNC_SOUT_BIT;
      else
         SYNC_SOUT_OUT &= ~SYNC_SOUT_BIT;
   
      // Pulse SRDY to indicate that the bit is ready for reading for ANT
      SYNC_SRDY_PULSE();
	
      // Wait for the SCLK rising edge that will indicate that the bit has been read
      while (!bSCLK_Detected);
      bSCLK_Detected = FALSE;                         // Clear the interrupt status flag

      ucBitMask <<= 1;                                // select the next bit

   } while (ucBitMask);	                              // Write the entire byte
}

