/** \file Main_Demo2.c
* 
* Example implementation of an ANT relay device using 
* MSP430F2013 microcontroller and IAR Kickstart for MSP430 V3
* 	
* $Name:  $
*/
/*******************************************************************************
* 
* Copyright (c) 2006 Dynastream Innovations Inc.
* THE FOLLOWING EXAMPLE CODE IS INTENDED FOR LIMITED CIRCULATION ONLY.
* 
* Please forward all questions regarding this code to ANT Technical Support.
* 
* Dynastream Innovations Inc.
* 228 River Avenue
* Cochrane, Alberta, Canada
* T4C 2C1
* 
* (P) (403) 932-9292
* (F) (403) 932-6521
* (TF) 1-866-932-9292
* (E) support@thisisant.com
* 
* www.thisisant.com
*******************************************************************************/

#include "ANTInterface.h"
#include "System.h"
#include "timer.h"
#include "antdefines.h"
#include "antmessage.h"
#include "types.h"
#include "Atod.h"
#include "Config.h"
#include "BitSyncSerial.h"


///////////////////////////////////////////////////////////////////////////////
// ANT Network Setup definitions
///////////////////////////////////////////////////////////////////////////////
// Network Settings
#define DEFAULT_NETWORK               ((UCHAR) 0x00)

// RF Frequency
#define DEFAULT_RF_FREQ               ((UCHAR) 0x42)     ///< decimal 66 (2466 MHz)
#define RELAY_RF_FREQ                 ((UCHAR) 0x4C)     ///< decimal 76 (2476 MHz)

// Transmission Type
#define TRANS_TYPE                    ((UCHAR) 0x03)    

// Device Types
#define DEVICE_TYPE                   ((UCHAR) 0x01)	

// Device Number (Pre-defined, no Device pairing supported)
#define DEVICE_NUM                    ((USHORT) 0x0031)  ///< decimal 49
#define DEVICE_NUM_MSB                ((UCHAR) 0x00)
#define DEVICE_NUM_LSB                ((UCHAR) 0x31)

// Message Periods
#define MSG_PERIOD_8HZ                ((USHORT) 0x1000)	 ///< decimal 4096 (8Hz)
#define MSG_PERIOD_8HZ_MSB            ((UCHAR) 0x10)	 ///< decimal 4096 (8Hz)
#define MSG_PERIOD_8HZ_LSB            ((UCHAR) 0x00)	 ///< decimal 4096 (8Hz)

// Channel Number
#define CHANNEL_0                     ((UCHAR) 0x00)
#define CHANNEL_1                     ((UCHAR) 0x01)


/////////////////////////////////////////////////////////////////////////////
// STATIC ANT Configuration Block
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Channel 0 Configuration Commands
static const UCHAR aucANTAssignChannel0[]    = {MESG_ASSIGN_CHANNEL_SIZE, MESG_ASSIGN_CHANNEL_ID, CHANNEL_0, CHANNEL_TYPE_MASTER, DEFAULT_NETWORK};               // Channel 0, Master Channel, Network 0 
static const UCHAR aucANTSetChannelID0[]     = {MESG_CHANNEL_ID_SIZE, MESG_CHANNEL_ID_ID, CHANNEL_0, DEVICE_NUM_LSB, DEVICE_NUM_MSB, DEVICE_TYPE, TRANS_TYPE};    // Channel 0, Device ID 49, Device Type 1, Man ID 3
static const UCHAR aucANTSetChannelPeriod0[] = {MESG_CHANNEL_MESG_PERIOD_SIZE, MESG_CHANNEL_MESG_PERIOD_ID, CHANNEL_0, MSG_PERIOD_8HZ_LSB, MSG_PERIOD_8HZ_MSB};   // Channel 0, Mesg Period 4 Hz
static const UCHAR aucANTSetChannelRFFreq0[] = {MESG_CHANNEL_RADIO_FREQ_SIZE, MESG_CHANNEL_RADIO_FREQ_ID, CHANNEL_0, DEFAULT_RF_FREQ};                            // Channel 0, RF Freq 2466 MHz
static const UCHAR aucANTOpenChannel0[]      = {MESG_OPEN_CHANNEL_SIZE, MESG_OPEN_CHANNEL_ID, CHANNEL_0};                                                         // Open Channel 0

// Channel 0 Configuration Command List
static const USHORT apucChannelConfig0[] ={  (USHORT)aucANTAssignChannel0,\
                                             (USHORT)aucANTSetChannelID0,\
                                             (USHORT)aucANTSetChannelPeriod0,\
                                             (USHORT)aucANTSetChannelRFFreq0,\
                                             (USHORT)aucANTOpenChannel0\
                                          };

/////////////////////////////////////////////////////////////////////////////
// Channel 1 Configuration Commands
static const UCHAR aucANTAssignChannel1[]    = {MESG_ASSIGN_CHANNEL_SIZE, MESG_ASSIGN_CHANNEL_ID, CHANNEL_1, CHANNEL_TYPE_SLAVE, DEFAULT_NETWORK};                ///< Channel 1, Master Channel, Network 0 
static const UCHAR aucANTSetChannelID1[]     = {MESG_CHANNEL_ID_SIZE, MESG_CHANNEL_ID_ID, CHANNEL_1, DEVICE_NUM_LSB, DEVICE_NUM_MSB, DEVICE_TYPE, TRANS_TYPE};    ///< Channel 1, Device ID 49, Device Type 1, Man ID 3
static const UCHAR aucANTSetChannelPeriod1[] = {MESG_CHANNEL_MESG_PERIOD_SIZE, MESG_CHANNEL_MESG_PERIOD_ID, CHANNEL_1, MSG_PERIOD_8HZ_LSB, MSG_PERIOD_8HZ_MSB};   ///< Channel 1, Mesg Period 4 Hz
static const UCHAR aucANTSetChannelRFFreq1[] = {MESG_CHANNEL_RADIO_FREQ_SIZE, MESG_CHANNEL_RADIO_FREQ_ID, CHANNEL_1, RELAY_RF_FREQ};                              ///< Channel 1, RF Freq 2476 MHz
static const UCHAR aucANTOpenChannel1[]      = {MESG_OPEN_CHANNEL_SIZE, MESG_OPEN_CHANNEL_ID, CHANNEL_1};                                                         ///< Open Channel 1

// Channel 1 Configuration Command List
static const USHORT apucChannelConfig1[] ={  (USHORT)aucANTAssignChannel1,\
                                             (USHORT)aucANTSetChannelID1,\
                                             (USHORT)aucANTSetChannelPeriod1,\
                                             (USHORT)aucANTSetChannelRFFreq1,\
                                             (USHORT)aucANTOpenChannel1\
                                          };

/////////////////////////////////////////////////////////////////////////////
// Channel N Configuration Commands

// Next channel configuration block


/////////////////////////////////////////////////////////////////////////////
// Total Channel lists
#define CC0_SIZE ((UCHAR)sizeof(apucChannelConfig0)/sizeof(USHORT))             ///< Number of commands in channel 0 config block
#define CC1_SIZE ((UCHAR)sizeof(apucChannelConfig1)/sizeof(USHORT))             ///< Number of commands in channel 1 config block

const UCHAR aucChannelConfigSize[] = { CC0_SIZE, CC1_SIZE };                    ///< Channel configuration blocks sizes (all channels)
const USHORT appucChannelConfig[] = { (USHORT)apucChannelConfig0, (USHORT)apucChannelConfig1 }; ///< Channel configuration list (all channels)

const UCHAR ucNumChannels = 2;                                                  ///< Number of channels defined


/////////////////////////////////////////////////////////////////////////////
// Local functions

static void Main_ProcessANTEvent(void);
static void Main_CopyBuffer(UCHAR *pucDest, UCHAR *pucSource);


/////////////////////////////////////////////////////////////////////////////
// Static Data

__no_init static   UCHAR aucTxMesgBuffer[MESG_BUFFER_SIZE]; ///< transmit message buffer
__no_init static   UCHAR *pucTxBuffer;                      ///< transmit message pointer
__no_init volatile USHORT usLowPowerMode;                   ///< low power mode control


/////////////////////////////////////////////////////////////////////////////
///================================= Main ===================================
///
/// The Main loop is responsible triggering the message and ANT event handling 
/// as well as putting the CPU to sleep when there is nothing to do. 
///
/////////////////////////////////////////////////////////////////////////////
void main(void)
{
   // Intialize all modules
   System_DisableWatchdog();              // Disable the watchdog
   System_Init();                         // Setup the clock 
   Timer_Init();                          // Setup the timer
   AtoD_Init();                           // Setup A/D
   ANTInterface_Init();                   // initialize the ANT interface

   // Optional Setup Ext 32 kHz clock 
   // P1DIR |= BIT0;                      // Set P1.0 Direction to Output
   // P1SEL |= BIT0;                      // Set P1.0 Selection to ACLK   

   // Optional LED control
   LED_DIR |= LED_BIT;                    // enable the LED
   LED_OUT |= LED_BIT;                    // Turn on the LED

   // Low power wake up functionality
   SYNC_SEN_IES |= SYNC_SEN_BIT;          // interrupt on high-to-low transition
   SYNC_SEN_IFG &= ~SYNC_SEN_BIT;         // reset the interrupt flag
   SYNC_SEN_IE  |= SYNC_SEN_BIT;          // enable the SEN interrupt to allow wake up from sleep                                       
  
   pucTxBuffer = NULL;                    // initialize the tx buffer pointer to null (no message to send)

   __enable_interrupt();                  // enable global interrupts
   Timer_DelayTime(50000);                // delay for 500 ms to allow power level and DCO to settle before serial communication 
   // ***********************
   // Begin ANT Communication
   // ***********************
   while (!ANTInterface_ChannelConfig((USHORT *)appucChannelConfig, (UCHAR *)aucChannelConfigSize, ucNumChannels));  // Configure the channels
   // Channel Setup is complete

   LED_OUT &= ~LED_BIT;                   // Turn off the LED
 
   // Main Loop
   while (1)                              // Now handle the incoming messages forever
   {
      usLowPowerMode = LPM4_bits;         // initialize the low power mode

      ANTInterface_MesgProcess(pucTxBuffer); // process pending serial port messaging

      pucTxBuffer = NULL;                 // reset the transmit message

      Main_ProcessANTEvent();             // process ANT messaging     

      _BIS_SR(usLowPowerMode);            // go to sleep if we can
   }	
}
//------------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
/// Main_ProcessANTEvent
///
/// This function is used to handle events/messages received from the ANT chip
///
/////////////////////////////////////////////////////////////////////////////
void Main_ProcessANTEvent(void)
{
   UCHAR *pucEventBuffer = ANTInterface_GetPendingEvent();

   if (pucEventBuffer == NULL)
      return;
   
   switch (pucEventBuffer[BUFFER_INDEX_MESG_ID])	                         // switch the current event that needs processing
   {
      case MESG_BROADCAST_DATA_ID:
      case MESG_ACKNOWLEDGED_DATA_ID:      
         switch (pucEventBuffer[BUFFER_INDEX_CHANNEL_NUM])
         {
            case CHANNEL_0:			                                 // channel 0
               // Forward data onto other channel
               pucTxBuffer = aucTxMesgBuffer;                        // initialize the buffer pointer
               Main_CopyBuffer(pucTxBuffer, pucEventBuffer);         // copy the message
               pucTxBuffer[BUFFER_INDEX_CHANNEL_NUM] = CHANNEL_1;    // overwrite the channel number to be the forwarded channel number
               usLowPowerMode = 0;                                   // prevent the system from going to sleep
               break;

            case CHANNEL_1:			                                 // channel 1
               // Forward data onto other channel
               pucTxBuffer = aucTxMesgBuffer;                        // initialize the buffer pointer
               Main_CopyBuffer(pucTxBuffer, pucEventBuffer);         // copy the message
               pucTxBuffer[BUFFER_INDEX_CHANNEL_NUM] = CHANNEL_0;    // overwrite the channel number to be the forwarded channel number
               usLowPowerMode = 0;                                   // prevent the system from going to sleep
               break;							         
         }
         break;
    
      // Handle any other messages here

      default:
         break;
   }
}

//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// Main_CopyBuffer
///
/// This function is used to copy message data from one buffer to another.
///
/// \param pucDest - pointer to the destination messge buffer
/// \param pucSource - pointer to the source message buffer
///
/////////////////////////////////////////////////////////////////////////////
static void Main_CopyBuffer(UCHAR *pucDest, UCHAR *pucSource)
{
   UCHAR i;

   for (i=0; i<MESG_BUFFER_SIZE; i++)
      *pucDest++ = *pucSource++;
}

//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// INT_PORT1
///
/// This function handles all port 1 interrupts, these include:
/// -SCLK rising edge
/// -SEN falling edge 
///
/////////////////////////////////////////////////////////////////////////////
#pragma vector=PORT1_VECTOR
__interrupt void INT_PORT1(void)
{
   while (P1IFG)
   {
      if (SYNC_SCLK_IFG & SYNC_SCLK_BIT)
      {
        SYNC_SCLK_IFG &= ~SYNC_SCLK_BIT; 
        bSCLK_Detected = TRUE;
      }  
      else if (SYNC_SEN_IFG & SYNC_SEN_BIT)
      {
        //////////////////////////////////////////////////////////////////////////////////////////////////
        // when interrupt exits, microprocessor must come out of sleep and run main, to process ANT events
        //////////////////////////////////////////////////////////////////////////////////////////////////
        usLowPowerMode = 0;                                // Prevent the Main loop from going to sleep if it was just about too (prevents a race condition)
        SYNC_SEN_IFG &= ~SYNC_SEN_BIT;   
        __low_power_mode_off_on_exit();                    // this is an MSP430 / IAR specific function to cause the microprocessor to wake up when interrupt exits        
      }        
      else
      {
         P1IFG = 0;                                        // Clear all other port 1 interrupts.
      }
   }   
}

