/** \file Main_Demo3.c
* 
* Example implementation of an ASC Slave ANT Temperature Sensor device using 
* MSP430F2013 microcontroller and IAR Kickstart for MSP430 V3
* 	
* $Name:  $
*/
/*******************************************************************************
* Copyright (c) 2006 Dynastream Innovations Inc.
* THE FOLLOWING EXAMPLE CODE IS INTENDED FOR LIMITED CIRCULATION ONLY.
* 
* Please forward all questions regarding this code to ANT Technical Support.
* 
* Dynastream Innovations Inc.
* 228 River Avenue
* Cochrane, Alberta, Canada
* T4C 2C1
* 
* (P) (403) 932-9292
* (F) (403) 932-6521
* (TF) 1-866-932-9292
* (E) support@thisisant.com
* 
* www.thisisant.com
*******************************************************************************/

#include "ANTInterface.h"
#include "System.h"
#include "timer.h"
#include "antdefines.h"
#include "antmessage.h"
#include "types.h"
#include "Atod.h"
#include "Config.h"
#include "BitSyncSerial.h"


///////////////////////////////////////////////////////////////////////////////
// ANT Network Setup definitions
///////////////////////////////////////////////////////////////////////////////

// Network Settings
#define DEFAULT_NETWORK               ((UCHAR) 0x00)

// RF Frequency
#define DEFAULT_RF_FREQ               ((UCHAR) 0x42)     ///< decimal 66 (2466 MHz)

// Transmission Type
#define TRANS_TYPE                    ((UCHAR) 0x07)    

// Device Types
#define DEVICE_TYPE                   ((UCHAR) 0x01)	

// Device Number (Pre-defined, no Device pairing supported)
#define DEVICE_NUM                    ((USHORT) 0x0031)  ///< decimal 49
#define DEVICE_NUM_MSB                ((UCHAR) 0x00)
#define DEVICE_NUM_LSB                ((UCHAR) 0x31)

// Message Periods
#define MSG_PERIOD_8HZ                ((USHORT) 0x1000)	 ///< decimal 4096 (8Hz)
#define MSG_PERIOD_8HZ_MSB            ((UCHAR) 0x10)	 ///< decimal 4096 (8Hz)
#define MSG_PERIOD_8HZ_LSB            ((UCHAR) 0x00)	 ///< decimal 4096 (8Hz)

// Channel Number
#define CHANNEL_0                     ((UCHAR) 0x00)

// LED definititions
#define LED_STATE_INDEX               ((UCHAR)(BUFFER_INDEX_MESG_DATA + 7))
#define LED_MASK_INDEX                ((UCHAR)(BUFFER_INDEX_MESG_DATA + 6))

/////////////////////////////////////////////////////////////////////////////
// STATIC ANT Configuration Block
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Channel 0 Configuration Commands
static const UCHAR aucANTAssignChannel0[]    = {MESG_ASSIGN_CHANNEL_SIZE, MESG_ASSIGN_CHANNEL_ID, CHANNEL_0, CHANNEL_TYPE_SHARED_SLAVE, DEFAULT_NETWORK};         ///< Channel 0, Master Channel, Network 0 
static const UCHAR aucANTSetChannelID0[]     = {MESG_CHANNEL_ID_SIZE, MESG_CHANNEL_ID_ID, CHANNEL_0, DEVICE_NUM_LSB, DEVICE_NUM_MSB, DEVICE_TYPE, TRANS_TYPE};    ///< Channel 0, Device ID 49, Device Type 1, Man ID 3
static const UCHAR aucANTSetChannelPeriod0[] = {MESG_CHANNEL_MESG_PERIOD_SIZE, MESG_CHANNEL_MESG_PERIOD_ID, CHANNEL_0, MSG_PERIOD_8HZ_LSB, MSG_PERIOD_8HZ_MSB};   ///< Channel 0, Mesg Period 4 Hz
static const UCHAR aucANTSetChannelRFFreq0[] = {MESG_CHANNEL_RADIO_FREQ_SIZE, MESG_CHANNEL_RADIO_FREQ_ID, CHANNEL_0, DEFAULT_RF_FREQ};                            ///< Channel 0, RF Freq 2466 MHz
static const UCHAR aucANTOpenChannel0[]      = {MESG_OPEN_CHANNEL_SIZE, MESG_OPEN_CHANNEL_ID, CHANNEL_0};                                                         ///< Open Channel 0

// Channel 0 Configuration Command List
static const USHORT apucChannelConfig0[] ={  (USHORT)aucANTAssignChannel0,\
                                             (USHORT)aucANTSetChannelID0,\
                                             (USHORT)aucANTSetChannelPeriod0,\
                                             (USHORT)aucANTSetChannelRFFreq0,\
                                             (USHORT)aucANTOpenChannel0\
                                          };

/////////////////////////////////////////////////////////////////////////////
// Channel N Configuration Commands

// Next channel configuration block


/////////////////////////////////////////////////////////////////////////////
// Total Channel lists
#define CC0_SIZE ((UCHAR)sizeof(apucChannelConfig0)/sizeof(USHORT))             ///< Number of commands in channel 0 config block

const UCHAR aucChannelConfigSize[] = { CC0_SIZE };                              ///< Channel configuration blocks sizes (all channels)
const USHORT appucChannelConfig[] = { (USHORT)apucChannelConfig0 };             ///< Channel configuration list (all channels)

const UCHAR ucNumChannels = 1;                                                  ///< Number of channels defined  


///////////////////////////////////////////////////////////////////////
// Auto SC Config Command Definitions
// These definitions should compliment the stand alone data type definitions
///////////////////////////////////////////////////////////////////////
#define SHARED_ADDRESS_SEARCHING       ((USHORT)0xFFFF)
#define SHARED_ADDRESS_AQUIRING        ((USHORT)0xFFFE)
#define SHARED_ADDRESS_AQUIRING_LSB    ((UCHAR)0xFE)
#define SHARED_ADDRESS_AQUIRING_MSB    ((UCHAR)0xFF)
#define SHARED_CHANNEL_UNIQUE_ID       ((UCHAR)(BUFFER_INDEX_MESG_DATA + 4))
#define SHARED_CHANNEL_NEW_ADDRESS     ((UCHAR)(BUFFER_INDEX_MESG_DATA + 6))

typedef enum
{
   NO_STATE = 0,
   SEARCHING_FOR_ADDRESS,
   REQUESTING_ADDRESS,
   WAITING_FOR_CONFIRM,
   CONFIRMING_ADDRESS,
   ADDRESS_SET,
   OPERATING

} AUTO_SHARED_STATE;

static const UCHAR aucSerialNumber[] = {0x04,0x00,0x00,0x00};                    ///< !!Must be modified for each separate device to distinguish them


/////////////////////////////////////////////////////////////////////////////
// Local functions

static void Main_ProcessANTEvent(void);
static void Main_CopyBuffer(UCHAR *pucDest, UCHAR *pucSource);
static void Main_SetSharedAddress(UCHAR *pucMesgBuffer, USHORT usAddress, UCHAR ucChannelNum);


/////////////////////////////////////////////////////////////////////////////
// Static Data

__no_init static   UCHAR aucTxMesgBuffer[MESG_BUFFER_SIZE];                      ///< transmit message buffer
__no_init static   UCHAR *pucTxBuffer;                                           ///< transmit message pointer
__no_init volatile USHORT usLowPowerMode;                                        ///< low power mode control

__no_init static USHORT usSharedAddress;                                         ///< auto shared address number
__no_init static AUTO_SHARED_STATE eSharedState;                                 ///< auto shared address state

/////////////////////////////////////////////////////////////////////////////
///================================= Main ===================================
///
/// The Main loop is responsible triggering the message and ANT event handling 
/// as well as putting the CPU to sleep when there is nothing to do. 
///
/////////////////////////////////////////////////////////////////////////////
void main(void)
{
   // Intialize all modules
   System_DisableWatchdog();                                                     // Disable the watchdog
   System_Init();                                                                // Setup the clock 
   Timer_Init();                                                                 // Setup the timer
   AtoD_Init();                                                                  // Setup A/D
   ANTInterface_Init();                                                          // initialize the ANT interface

   // Optional Setup Ext 32 kHz clock 
   // P1DIR |= BIT0;                                                             // Set P1.0 Direction to Output
   // P1SEL |= BIT0;                                                             // Set P1.0 Selection to ACLK   
     
   // Optional LED control
   LED_DIR |= LED_BIT;                                                           // enable the LED
   LED_OUT |= LED_BIT;                                                           // Turn on the LED
   
   // Low power wake up functionality
   SYNC_SEN_IES |= SYNC_SEN_BIT;                                                 // interrupt on high-to-low transition
   SYNC_SEN_IFG &= ~SYNC_SEN_BIT;                                                // reset the interrupt flag
   SYNC_SEN_IE  |= SYNC_SEN_BIT;                                                 // enable the SEN interrupt to allow wake up from sleep                                       

   pucTxBuffer = NULL;                                                           // initialize the tx buffer pointer to null (no message to send)

   __enable_interrupt();                                                         // enable global interrupts
   // ***********************
   // Begin ANT Communication
   // ***********************
   while (!ANTInterface_ChannelConfig((USHORT *)appucChannelConfig, (UCHAR *)aucChannelConfigSize, ucNumChannels));  // Configure the channels
   // Channel Setup is complete

   LED_OUT        &= ~LED_BIT;                                                   // Turn off the LED
   usSharedAddress = 0;                                                          // initialize the shared address
   eSharedState    = SEARCHING_FOR_ADDRESS;                                      // initialize the shared address state
   pucTxBuffer     = aucTxMesgBuffer;                                            // initialize the tx buffer pointer to null (no message to send)

   Main_SetSharedAddress(pucTxBuffer, SHARED_ADDRESS_SEARCHING, CHANNEL_0);      // initialize the shared address in the ANT device


   // Main Loop
   while (1)                                                                     // Now handle the incoming messages forever
   {
      usLowPowerMode = LPM4_bits;                                                // initialize the low power mode

      ANTInterface_MesgProcess(pucTxBuffer);                                     // process pending serial port messaging
      pucTxBuffer = NULL;                                                        // reset the transmit message

      Main_ProcessANTEvent();                                                    // process ANT messaging     

      _BIS_SR(usLowPowerMode);                                                   // go to sleep if we can
   }	
}
//------------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
/// Main_ProcessANTEvent
///
/// This function is used to handle events/messages received from the ANT chip
///
/////////////////////////////////////////////////////////////////////////////
void Main_ProcessANTEvent(void)
{
   AUTO_SHARED_STATE eNewSharedState;
   UCHAR *pucEventBuffer;
   
   pucEventBuffer  = ANTInterface_GetPendingEvent();
   eNewSharedState = NO_STATE;

   if (pucEventBuffer == NULL)
      return;

   switch (pucEventBuffer[BUFFER_INDEX_CHANNEL_NUM])                             // switch on channel number
   {
      case CHANNEL_0:                                                            // channel 0 processing
         switch (pucEventBuffer[BUFFER_INDEX_MESG_ID])	                         // switch on message type
         {
            case MESG_BROADCAST_DATA_ID:
            case MESG_ACKNOWLEDGED_DATA_ID:      
               switch(pucEventBuffer[BUFFER_INDEX_SHARED_DATA_TYPE])             // switch on shared channel control state
               {                
                  case SHARED_CMD_SLOT_AVALIBLE:                                 // Check if current message indicates slot is available
                     if (eSharedState == SEARCHING_FOR_ADDRESS)                  // If we are looking for a slot
                     {
                        usSharedAddress  = (USHORT)pucEventBuffer[SHARED_CHANNEL_NEW_ADDRESS];          // read out the new address
                        usSharedAddress |= ((USHORT)pucEventBuffer[SHARED_CHANNEL_NEW_ADDRESS + 1] << 8);

                        pucEventBuffer[BUFFER_INDEX_MESG_ID]          = MESG_ACKNOWLEDGED_DATA_ID;      // send acknowledged message
                        pucEventBuffer[BUFFER_INDEX_SHARED_DATA_TYPE] = SHARED_CMD_COMMAND_REQUEST_TO_ACQUIRE;  // Indicate in message to be sent that we wish to acquire the slot
                        pucEventBuffer[SHARED_CHANNEL_UNIQUE_ID]      = aucSerialNumber[0];
                        pucEventBuffer[SHARED_CHANNEL_UNIQUE_ID + 1]  = aucSerialNumber[1];
                        pucEventBuffer[SHARED_CHANNEL_UNIQUE_ID + 2]  = aucSerialNumber[2];
                        pucEventBuffer[SHARED_CHANNEL_UNIQUE_ID + 3]  = aucSerialNumber[3];

                        Main_CopyBuffer(aucTxMesgBuffer, pucEventBuffer);        // copy modified response buffer into tx buffer
                        
                        eNewSharedState = REQUESTING_ADDRESS;                    // set next state  
                     }
                     break;

                  case SHARED_CMD_BUSY_ACQUIRING:
                     if (eSharedState == WAITING_FOR_CONFIRM)                    // Check if current message indicates slot is waiting for confirmation
                     {
                        if ((pucEventBuffer[SHARED_CHANNEL_UNIQUE_ID] == aucSerialNumber[0]) &&
                           (pucEventBuffer[SHARED_CHANNEL_UNIQUE_ID+1] == aucSerialNumber[1]) &&
                           (pucEventBuffer[SHARED_CHANNEL_UNIQUE_ID+2] == aucSerialNumber[2]) &&
                           (pucEventBuffer[SHARED_CHANNEL_UNIQUE_ID+3] == aucSerialNumber[3]))            // if the message was for us
                        {
                           pucEventBuffer[BUFFER_INDEX_MESG_ID]            = MESG_ACKNOWLEDGED_DATA_ID;   // send acknowledged message
                           pucEventBuffer[BUFFER_INDEX_SHARED_ADDRESS_LSB] = SHARED_ADDRESS_AQUIRING_LSB; // Set shared address to 0xFFFE
                           pucEventBuffer[BUFFER_INDEX_SHARED_ADDRESS_MSB] = SHARED_ADDRESS_AQUIRING_MSB;
                           pucEventBuffer[BUFFER_INDEX_SHARED_DATA_TYPE]   = SHARED_CMD_CONFIRM_ACQUIRED; // confirm that we want to acquire this slot

                           Main_CopyBuffer(aucTxMesgBuffer, pucEventBuffer);     // copy modified response buffer into tx buffer
                           
                           eNewSharedState = CONFIRMING_ADDRESS;
                        }
                        else                                                     // otherwise, begin searching again
                        {
                           eNewSharedState = SEARCHING_FOR_ADDRESS;
                        }
                     }
                     break;
            
                  default:
                     if (pucEventBuffer[BUFFER_INDEX_SHARED_ADDRESS_LSB] || pucEventBuffer[BUFFER_INDEX_SHARED_ADDRESS_MSB]) // if the shared address is non zero then it must match the one we set
                     {
                        if (pucEventBuffer[BUFFER_INDEX_SHARED_DATA_TYPE] == SHARED_TYPE_DIGITAL)     // Check if we received a set LED state message
                           LED_OUT = (LED_OUT & ~LED_BIT) | (pucEventBuffer[LED_STATE_INDEX] & pucEventBuffer[LED_MASK_INDEX] & LED_BIT);  // Set LED state
                  
                        AtoD_SetTempMesg(aucTxMesgBuffer, CHANNEL_0);            // create an AtoD message for channel 0 to transmit                     
                        aucTxMesgBuffer[BUFFER_INDEX_SHARED_ADDRESS_LSB] = pucEventBuffer[BUFFER_INDEX_SHARED_ADDRESS_LSB]; // set the correct shared address 
                        aucTxMesgBuffer[BUFFER_INDEX_SHARED_ADDRESS_MSB] = pucEventBuffer[BUFFER_INDEX_SHARED_ADDRESS_MSB]; // set the correct shared address
                        
                        eNewSharedState = OPERATING;
                     }
                     break;
               }
               break;
          
            case MESG_RESPONSE_EVENT_ID:
               switch (pucEventBuffer[BUFFER_INDEX_RESPONSE_CODE])
               {
                  case EVENT_TRANSFER_TX_COMPLETED:
                     switch(eSharedState)
                     {
                        case REQUESTING_ADDRESS:                                 // if we were requesting an address
                           Main_SetSharedAddress(aucTxMesgBuffer, SHARED_ADDRESS_AQUIRING, CHANNEL_0);
                           eNewSharedState = WAITING_FOR_CONFIRM;
                           break;

                        case CONFIRMING_ADDRESS:                                 // we confirmed that we get the address
                           Main_SetSharedAddress(aucTxMesgBuffer, usSharedAddress, CHANNEL_0);
                           eNewSharedState = ADDRESS_SET;                        // we got the address                 
                           break;

                        default:
                           break;
                     }
                     break;

                  case EVENT_TRANSFER_TX_FAILED:  
                     switch(eSharedState)                                        // if we were tyring to acquire a slot, begin searching again
                     {
                        case REQUESTING_ADDRESS:                 
                        case CONFIRMING_ADDRESS:
                           eNewSharedState = SEARCHING_FOR_ADDRESS;
                           break;

                        default:
                           break;
                     }

                  case EVENT_RX_SEARCH_TIMEOUT:
                     ANTInterface_MesgProcess((UCHAR *)aucANTOpenChannel0);      // Re-open channel
                     eNewSharedState = SEARCHING_FOR_ADDRESS;
                     break;

                  case EVENT_RX_FAIL_GO_TO_SEARCH:                               // only applies in TI Module case, AP1 does not support this event
                     eNewSharedState = SEARCHING_FOR_ADDRESS;
                     break;

                  // add other response type handling here

                  default:
                     break;
               }             
               break;

            // Handle any other messages here

            default:
               break;
         }
         
         if (eNewSharedState)                                                    // we have a new shared state
         {
            eSharedState   = eNewSharedState;                                    // update the shared state
            pucTxBuffer    = aucTxMesgBuffer;                                    // initialize the buffer pointer
            usLowPowerMode = 0;                                                  // prevent the system from going to sleep

            if (eNewSharedState == SEARCHING_FOR_ADDRESS)                        // if we are going into searching state
               Main_SetSharedAddress(aucTxMesgBuffer, SHARED_ADDRESS_SEARCHING, CHANNEL_0); // set search state message
         }
         break;
      // handle other channel processing here

      default:
         break;
   }
}

//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// Main_CopyBuffer
///
/// This function is used to copy message data from one buffer to another.
///
/// \param pucDest - pointer to the destination messge buffer
/// \param pucSource - pointer to the source message buffer
/////////////////////////////////////////////////////////////////////////////
static void Main_CopyBuffer(UCHAR *pucDest, UCHAR *pucSource)
{
   UCHAR i;

   for (i=0; i<MESG_BUFFER_SIZE; i++)
      *pucDest++ = *pucSource++;
}

//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// Main_SetSharedAddress
///
/// This function is used to build a broadcast data message with a specific 
/// shared address and channel number.
///
/// \param pucMesgBuffer - Buffer where the message will be written
/// \param usAddress - Shared address the message will have
/// \param ucChannelNum - The ANT channel the message is ment for
///
/////////////////////////////////////////////////////////////////////////////
static void Main_SetSharedAddress(UCHAR *pucMesgBuffer, USHORT usAddress, UCHAR ucChannelNum)
{
   *pucMesgBuffer++ = MESG_DATA_SIZE;
   *pucMesgBuffer++ = MESG_BROADCAST_DATA_ID;
   *pucMesgBuffer++ = ucChannelNum;
   *pucMesgBuffer++ = (UCHAR)usAddress;
   *pucMesgBuffer++ = (UCHAR)(usAddress >> 8);   
   *pucMesgBuffer++ = 0;
   *pucMesgBuffer++ = 0;
   *pucMesgBuffer++ = 0;
   *pucMesgBuffer++ = 0;
   *pucMesgBuffer++ = 0;
   *pucMesgBuffer   = 0;
}

//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// INT_PORT1
///
/// This function handles all port 1 interrupts, these include:
/// -SCLK rising edge
/// -SEN falling edge 
///
/////////////////////////////////////////////////////////////////////////////
#pragma vector=PORT1_VECTOR
__interrupt void INT_PORT1(void)
{
   while (P1IFG)
   {
      if (SYNC_SCLK_IFG & SYNC_SCLK_BIT)
      {
        SYNC_SCLK_IFG &= ~SYNC_SCLK_BIT; 
        bSCLK_Detected = TRUE;
      }  
      else if (SYNC_SEN_IFG & SYNC_SEN_BIT)
      {
        //////////////////////////////////////////////////////////////////////////////////////////////////
        // when interrupt exits, microprocessor must come out of sleep and run main, to process ANT events
        //////////////////////////////////////////////////////////////////////////////////////////////////
        usLowPowerMode = 0;                                                      // Prevent the Main loop from going to sleep if it was just about too (prevents a race condition)
        SYNC_SEN_IFG &= ~SYNC_SEN_BIT;   
        __low_power_mode_off_on_exit();                                          // this is an MSP430 / IAR specific function to cause the microprocessor to wake up when interrupt exits        
      }        
      else
      {
         P1IFG = 0;                                                              // Clear all other port 1 interrupts.
      }
   }   
}
