/** \file Main_Demo5.c
* 
* Example implementation of a fixed address shared channel slave ANT device using 
* MSP430F2013 microcontroller and IAR Kickstart for MSP430 V3
* 	
* $Name:  $
*/
/*******************************************************************************
* Copyright (c) 2006 Dynastream Innovations Inc.
* THE FOLLOWING EXAMPLE CODE IS INTENDED FOR LIMITED CIRCULATION ONLY.
* 
* Please forward all questions regarding this code to ANT Technical Support.
* 
* Dynastream Innovations Inc.
* 228 River Avenue
* Cochrane, Alberta, Canada
* T4C 2C1
* 
* (P) (403) 932-9292
* (F) (403) 932-6521
* (TF) 1-866-932-9292
* (E) support@thisisant.com
* 
* www.thisisant.com
*******************************************************************************/

#include "ANTInterface.h"
#include "System.h"
#include "timer.h"
#include "antdefines.h"
#include "antmessage.h"
#include "types.h"
#include "Atod.h"
#include "Config.h"
#include "BitSyncSerial.h"


///////////////////////////////////////////////////////////////////////////////
// ANT Network Setup definitions
///////////////////////////////////////////////////////////////////////////////

// Network Settings
#define DEFAULT_NETWORK               ((UCHAR) 0x00)

// RF Frequency
#define DEFAULT_RF_FREQ               ((UCHAR) 0x42)     ///< decimal 66 (2466 MHz)

// Transmission Type
#define TRANS_TYPE                    ((UCHAR) 0x05)    

// Device Types
#define DEVICE_TYPE                   ((UCHAR) 0x01)	

// Device Number (Pre-defined, no Device pairing supported)
#define DEVICE_NUM                    ((USHORT) 0x0031)  ///< decimal 49
#define DEVICE_NUM_MSB                ((UCHAR) 0x00)
#define DEVICE_NUM_LSB                ((UCHAR) 0x31)

// Message Periods
#define MSG_PERIOD_4HZ                ((USHORT) 0x2000)	 ///< decimal 8192 (4Hz)
#define MSG_PERIOD_4HZ_MSB            ((UCHAR) 0x20)	 ///< decimal 8192 (4Hz)
#define MSG_PERIOD_4HZ_LSB            ((UCHAR) 0x00)	 ///< decimal 8192 (4Hz)

// Channel Number
#define CHANNEL_0                     ((UCHAR) 0x00)

// Shared Address
#define SHARED_ADDRESS                ((USHORT)0x0001)

// LED data defines
#define LED_MESG_TYPE                 ((UCHAR) 0x01)
#define LED_STATE_INDEX               ((UCHAR)(BUFFER_INDEX_MESG_DATA + 7))


/////////////////////////////////////////////////////////////////////////////
// STATIC ANT Configuration Block
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Channel 0 Configuration Commands
static const UCHAR aucANTAssignChannel0[]    = {MESG_ASSIGN_CHANNEL_SIZE, MESG_ASSIGN_CHANNEL_ID, CHANNEL_0, CHANNEL_TYPE_SLAVE, DEFAULT_NETWORK};       ///< Channel 0, Master Channel, Network 0 
static const UCHAR aucANTSetChannelID0[]     = {MESG_CHANNEL_ID_SIZE, MESG_CHANNEL_ID_ID, CHANNEL_0, DEVICE_NUM_LSB, DEVICE_NUM_MSB, DEVICE_TYPE, TRANS_TYPE};  ///< Channel 0, Device ID 49, Device Type 1, Man ID 3
static const UCHAR aucANTSetChannelPeriod0[] = {MESG_CHANNEL_MESG_PERIOD_SIZE, MESG_CHANNEL_MESG_PERIOD_ID, CHANNEL_0, MSG_PERIOD_4HZ_LSB, MSG_PERIOD_4HZ_MSB}; ///< Channel 0, Mesg Period 4 Hz
static const UCHAR aucANTSetChannelRFFreq0[] = {MESG_CHANNEL_RADIO_FREQ_SIZE, MESG_CHANNEL_RADIO_FREQ_ID, CHANNEL_0, DEFAULT_RF_FREQ};                          ///< Channel 0, RF Freq 2466 MHz
static const UCHAR aucANTOpenChannel0[]      = {MESG_OPEN_CHANNEL_SIZE, MESG_OPEN_CHANNEL_ID, CHANNEL_0};     

// Channel 0 Configuration Command List
static const USHORT apucChannelConfig0[] ={  (USHORT)aucANTAssignChannel0,\
                                             (USHORT)aucANTSetChannelID0,\
                                             (USHORT)aucANTSetChannelPeriod0,\
                                             (USHORT)aucANTSetChannelRFFreq0,\
                                             (USHORT)aucANTOpenChannel0\
                                          };

/////////////////////////////////////////////////////////////////////////////
// Channel N Configuration Commands

// Next channel configuration block


/////////////////////////////////////////////////////////////////////////////
// Total Channel lists
#define CC0_SIZE ((UCHAR)sizeof(apucChannelConfig0)/sizeof(USHORT))             ///< Number of commands in channel 0 config block

const UCHAR aucChannelConfigSize[] = { CC0_SIZE };                              ///< Channel configuration blocks sizes (all channels)
const USHORT appucChannelConfig[] = { (USHORT)apucChannelConfig0 };             ///< Channel configuration list (all channels)

const UCHAR ucNumChannels = 1;                                                  ///< Number of channels defined


/////////////////////////////////////////////////////////////////////////////
// Local functions

static void Main_ProcessANTEvent(void);
static void Main_SetSharedAddress(UCHAR *pucMesgBuffer, USHORT usAddress, UCHAR ucChannelNum);


/////////////////////////////////////////////////////////////////////////////
// Static Data

__no_init static UCHAR aucTxMesgBuffer[MESG_MAX_SIZE];
__no_init static UCHAR *pucTxBuffer;
__no_init volatile USHORT usLowPowerMode;

/////////////////////////////////////////////////////////////////////////////
///================================= Main ===================================
///
/// The Main loop is responsible triggering the message and ANT event handling 
/// as well as putting the CPU to sleep when there is nothing to do. 
///
/////////////////////////////////////////////////////////////////////////////
void main(void)
{
   // Intialize all modules
   System_DisableWatchdog();              // Disable the watchdog
   System_Init();                         // Setup the clock 
   Timer_Init();                          // Setup the timer
   AtoD_Init();                           // Setup A/D
   ANTInterface_Init();                   // initialize the ANT interface

   // Optional Setup Ext 32 kHz clock 
   // P1DIR |= BIT0;                      // Set P1.0 Direction to Output
   // P1SEL |= BIT0;                      // Set P1.0 Selection to ACLK   

   // Optional LED control
   LED_DIR |= LED_BIT;                    // enable the LED
   LED_OUT |= LED_BIT;                    // Turn on the LED

   // Low power wake up functionality
   SYNC_SEN_IES |= SYNC_SEN_BIT;          // interrupt on high-to-low transition
   SYNC_SEN_IFG &= ~SYNC_SEN_BIT;         // reset the interrupt flag
   SYNC_SEN_IE  |= SYNC_SEN_BIT;          // enable the SEN interrupt to allow wake up from sleep                                       

   pucTxBuffer = NULL;                    // initialize the tx buffer pointer to null (no message to send)
   __enable_interrupt();                  // enable global interrupts
   Timer_DelayTime(50000);                // delay for 500 ms to allow power level and DCO to settle before serial communication 
   // ***********************
   // Begin ANT Communication
   // ***********************
   
   while (!ANTInterface_ChannelConfig((USHORT *)appucChannelConfig, (UCHAR *)aucChannelConfigSize, ucNumChannels));  // Configure the channels
   // Channel Setup is complete

   LED_OUT    &= ~LED_BIT;                // Turn off the LED
   pucTxBuffer = aucTxMesgBuffer;         // initialize the tx buffer pointer to null (no message to send)

   Main_SetSharedAddress(pucTxBuffer, SHARED_ADDRESS, CHANNEL_0); // initialize the shared address in the ANT device

   // Main Loop
   while (1)                              // Now handle the incoming messages forever
   {
      usLowPowerMode = LPM4_bits;         // initialize the low power mode

      ANTInterface_MesgProcess(pucTxBuffer); // process pending serial port messaging
      pucTxBuffer = NULL;
      Main_ProcessANTEvent();             // process ANT messaging
      
      _BIS_SR(usLowPowerMode);            // go to sleep if we can
   }	
}
//------------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
/// Main_ProcessANTEvent
///
/// This function is used to handle events/messages received from the ANT chip
///
/////////////////////////////////////////////////////////////////////////////
static void Main_ProcessANTEvent(void)
{
   static UCHAR ucLastButton = 0xFF; // State of the digital line last message.

   UCHAR *pucEventBuffer = ANTInterface_GetPendingEvent();

   if (pucEventBuffer == NULL)
      return;
   
   switch (pucEventBuffer[BUFFER_INDEX_MESG_ID]) // switch the current event that needs processing
   {
      case MESG_BROADCAST_DATA_ID:
      case MESG_ACKNOWLEDGED_DATA_ID:      
         switch (pucEventBuffer[BUFFER_INDEX_CHANNEL_NUM])
         {
            case CHANNEL_0:
               
               // Check if we received a set LED state message            			
               if (pucEventBuffer[BUFFER_INDEX_MESG_DATA] == MESG_SENSRCORE_DIGITAL_DATA) 
               {
                  // Toggle only on a HIGH to LOW transition, 
                  if((pucEventBuffer[LED_STATE_INDEX] & LED_BIT) == 0 && (pucEventBuffer[LED_STATE_INDEX] & LED_BIT) != ucLastButton)       
                     LED_OUT ^= LED_BIT;
                  
                  ucLastButton = pucEventBuffer[LED_STATE_INDEX];
               }
               usLowPowerMode = 0;                       // prevent the system from going to sleep
               pucTxBuffer    = aucTxMesgBuffer;         // initialize the buffer pointer
               AtoD_SetTempMesg(pucTxBuffer, CHANNEL_0); // create an AtoD temperature message for channel 0 to transmit                                         

               aucTxMesgBuffer[BUFFER_INDEX_SHARED_ADDRESS_LSB] = pucEventBuffer[BUFFER_INDEX_SHARED_ADDRESS_LSB];
               aucTxMesgBuffer[BUFFER_INDEX_SHARED_ADDRESS_MSB] = pucEventBuffer[BUFFER_INDEX_SHARED_ADDRESS_MSB];
               break;  

            // other channel receive processing done here
            default:
               break;
         }
         break;
      
      // Handle any other messages here
      default:
         break;
   }
}

//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// Main_SetSharedAddress
///
/// This function is used to build a broadcast data message with a specific 
/// shared address and channel number.
///
/// \param pucMesgBuffer - Buffer where the message will be written
/// \param usAddress - Shared address the message will have
/// \param ucChannelNum - The ANT channel the message is ment for
///
/////////////////////////////////////////////////////////////////////////////
static void Main_SetSharedAddress(UCHAR *pucMesgBuffer, USHORT usAddress, UCHAR ucChannelNum)
{
   *pucMesgBuffer++ = MESG_DATA_SIZE;
   *pucMesgBuffer++ = MESG_BROADCAST_DATA_ID;
   *pucMesgBuffer++ = ucChannelNum;
   *pucMesgBuffer++ = (UCHAR)usAddress;
   *pucMesgBuffer++ = (UCHAR)(usAddress >> 8);   
   *pucMesgBuffer++ = 0;
   *pucMesgBuffer++ = 0;
   *pucMesgBuffer++ = 0;
   *pucMesgBuffer++ = 0;
   *pucMesgBuffer++ = 0;
   *pucMesgBuffer   = 0;
}


//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
/// INT_PORT1
///
/// This function handles all port 1 interrupts, these include:
/// -SCLK rising edge
/// -SEN falling edge 
///
/////////////////////////////////////////////////////////////////////////////
#pragma vector=PORT1_VECTOR
__interrupt void INT_PORT1(void)
{
   while (P1IFG)
   {
      if (SYNC_SCLK_IFG & SYNC_SCLK_BIT)
      {
        SYNC_SCLK_IFG &= ~SYNC_SCLK_BIT; 
        bSCLK_Detected = TRUE;
      }  
      else if (SYNC_SEN_IFG & SYNC_SEN_BIT)
      {
        //////////////////////////////////////////////////////////////////////////////////////////////////
        // when interrupt exits, microprocessor must come out of sleep and run main, to process ANT events
        //////////////////////////////////////////////////////////////////////////////////////////////////
        usLowPowerMode = 0;                                // Prevent the Main loop from going to sleep if it was just about too (prevents a race condition)
        SYNC_SEN_IFG &= ~SYNC_SEN_BIT;   
        __low_power_mode_off_on_exit();                    // this is an MSP430 / IAR specific function to cause the microprocessor to wake up when interrupt exits        
      }        
      else
      {
         P1IFG = 0;                                        // Clear all other port 1 interrupts.
      }
   }   
}
